import os


class Paths:
    home: str
    data: str
    gui: str
    providers: str
    utils: str
    icon: str
    titlebar: str
    tabs: str


class SetPaths:
    def __init__(self) -> None:
        home = os.path.abspath(os.path.join(os.path.dirname(__file__), "../"))
        all_paths = {
            "home": home,
            "data": os.path.join(home, "data"),
            "gui": os.path.join(home, "gui"),
            "providers": os.path.join(home, "providers"),
            "utils": os.path.join(home, "utils"),
            "icon": os.path.join(home, "gui", "assets", "icon", "subsearch.ico"),
            "titlebar": os.path.join(home, "gui", "assets", "titlebar"),
            "tabs": os.path.join(home, "gui", "assets", "tabs"),
        }
        for k, v in all_paths.items():
            setattr(Paths, k, v)


class SubsearchPath:
    """base class for subsearch path"""


if __name__ != "__main__":
    __set_values__ = SetPaths()
    __home__: str = getattr(Paths, "home")
    __data__: str = getattr(Paths, "data")
    __gui__: str = getattr(Paths, "gui")
    __providers__: str = getattr(Paths, "providers")
    __utils__: str = getattr(Paths, "utils")
    __icon__: str = getattr(Paths, "icon")
    __titlebar__: str = getattr(Paths, "titlebar")
    __tabs__: str = getattr(Paths, "tabs")



from subsearch.data import __video__
from subsearch.providers import generic
from subsearch.providers.generic import BaseProvider, FormattedData
from subsearch.utils import log, string_parser
from subsearch.utils.raw_config import UserParameters
from subsearch.utils.string_parser import FileSearchParameters


class YifiSubtitles(BaseProvider):
    def __init__(self, parameters: FileSearchParameters, user_parameters: UserParameters):
        BaseProvider.__init__(self, parameters, user_parameters)
        self.scrape = YifySubtitlesScrape()
        self.logged_and_sorted: list[FormattedData] = []

    def parse_site_results(self):
        subtitle_data = self.scrape.get_subtitle(self.url_yifysubtitles, self.current_language)
        to_be_downloaded: dict[str, str] = {}
        to_be_sorted: list[FormattedData] = []
        for key, value in subtitle_data.items():
            pct_result = string_parser.get_pct_value(key, self.release)
            log.output(f"[{pct_result:>3}%  match]: {key}")
            formatted_data = generic.format_key_value_pct("yifysubtitles", key, value, pct_result)
            to_be_sorted.append(formatted_data)
            if self.is_threshold_met(key, pct_result) is False:
                continue
            if value in to_be_downloaded.values():
                continue
            to_be_downloaded[key] = value
        self.logged_and_sorted = generic.log_and_sort_list("subscene", to_be_sorted, self.pct_threashold)

        if len(to_be_downloaded) == 0:
            log.output(f"No subtitles to download for {self.release}")
            log.output("Done with tasks\n")
            return None

        download_info = generic.named_tuple_zip_data("yifysubtitles", __video__.tmp_directory, to_be_downloaded)
        log.output("Done with tasks\n")
        return download_info

    def _sorted_list(self):
        return self.logged_and_sorted


class YifySubtitlesScrape(YifiSubtitles):
    def __init__(self):
        ...

    def get_subtitle(self, url: str, current_language: str) -> dict[str, str]:
        doc = generic.get_lxml_doc(url)
        tag_tbody = doc.find("tbody")
        subtitles: dict[str, str] = {}
        while tag_tbody is None:
            self.to_many_requests()
            tag_tbody = doc.find("tbody")
        tbody_content = tag_tbody.contents[1::2]
        for class_a1 in tbody_content:
            subtitle_language = class_a1.contents[3].text
            if current_language.lower() != subtitle_language.lower():
                continue
            _releases = class_a1.contents[5].contents[1].contents[3::2]
            _releases_stripped = list(map(lambda release: release.strip(), _releases))
            _releases_no_space = list(map(lambda release: release.replace(" ", "."), _releases_stripped))
            _release_lower = list(map(lambda release: release.lower(), _releases_no_space))
            for release in _release_lower:
                subtitle_url = class_a1.contents[5].contents[1].attrs["href"].replace("subtitles", "subtitle")
                subtitles[release] = f"https://yifysubtitles.org{subtitle_url}.zip"
        return subtitles



import ctypes
import os
import time

from subsearch.data import __version__, __video__
from subsearch.gui import widget_menu
from subsearch.providers import opensubtitles, subscene, yifysubtitles
from subsearch.providers.generic import FormattedData
from subsearch.utils import (
    current_user,
    file_manager,
    log,
    raw_config,
    raw_registry,
    string_parser,
)


class BaseInitializer:
    def __init__(self) -> None:
        self.current_language = raw_config.get_config_key("current_language")
        self.languages = raw_config.get_config_key("languages")
        self.percentage = raw_config.get_config_key("percentage")
        self.rename_best_match = raw_config.get_config_key("rename_best_match")
        self.show_download_window = raw_config.get_config_key("show_download_window")
        self.show_terminal = raw_config.get_config_key("show_terminal")
        subtitle_type = raw_config.get_config_key("subtitle_type")
        self.hearing_impaired = subtitle_type["hearing_impaired"]
        self.non_hearing_impaired = subtitle_type["non_hearing_impaired"]
        self.providers = raw_config.get_config_key("providers")
        self.pro_subscene = self.providers["subscene_site"]
        self.pro_opensubtitles_rss = self.providers["opensubtitles_rss"]
        self.pro_opensubtitles_hash = self.providers["opensubtitles_hash"]
        self.pro_yifysubtitles = self.providers["yifysubtitles_site"]
        self.file_exist = True if __video__.name is not None else False
        self.file_hash = file_manager.get_hash(__video__.path)
        self.user_parameters = raw_config.UserParameters(
            current_language=self.current_language,
            languages=self.languages,
            hearing_impaired=self.hearing_impaired,
            non_hearing_impaired=self.non_hearing_impaired,
            percentage=self.percentage,
            show_download_window=self.show_download_window,
        )
        self.opensubtitles_hash_results = None
        self.opensubtitles_rss_results = None
        self.subscene_results = None
        self.yifysubtitles_results = None
        self.opensubtitles_sorted_list: list = []
        self.subscene_sorted_list: list = []
        self.yifysubtitles_sorted_list: list = []
        self.combined_list: list[FormattedData] = []
        self.opensubtitles_hash_downloads = 0
        self.opensubtitles_rss_downloads = 0
        self.subscene_downloads = 0
        self.yifysubtitles_downloads = 0
        self.ran_download_tab = False
        if self.file_exist:
            self.file_hash = file_manager.get_hash(__video__.path)
            self.parameters = string_parser.get_parameters(__video__.name, self.file_hash, self.user_parameters)
            log.parameters(self.parameters, self.user_parameters)

    def all_providers_disabled(self) -> bool:
        self.providers = raw_config.get_config_key("providers")
        if (
            self.pro_subscene is False
            and self.pro_opensubtitles_rss is False
            and self.pro_opensubtitles_hash is False
            and self.pro_yifysubtitles is False
        ):
            return True
        return False


class Steps(BaseInitializer):
    def __init__(self) -> None:
        self.start = time.perf_counter()
        BaseInitializer.__init__(self)
        ctypes.windll.kernel32.SetConsoleTitleW(f"subsearch - {__version__}")
        if current_user.got_key() is False:
            raw_config.set_default_json()
            raw_registry.add_context_menu()
        if __video__.tmp_directory is not None:
            if not os.path.exists(__video__.tmp_directory):
                os.mkdir(__video__.tmp_directory)
            if not os.path.exists(__video__.subs_directory):
                os.mkdir(__video__.subs_directory)
        if self.file_exist is False:
            widget_menu.open_tab("search")
            return None

        if " " in __video__.name:
            log.output("[Warning: Filename contains spaces]")

    def _provider_opensubtitles(self) -> None:
        if self.file_exist is False:
            return None
        if self.languages[self.current_language] == "N/A":
            log.output("\n[Searching on opensubtitles]")
            log.output(f"{self.current_language} not supported on opensubtitles\n")
            return None

        _opensubtitles = opensubtitles.OpenSubtitles(self.parameters, self.user_parameters)
        if self.pro_opensubtitles_hash and self.file_hash != "000000000000000000":
            log.output("\n[Searching on opensubtitles - hash]")
            self.opensubtitles_hash_results = _opensubtitles.parse_hash_results()
        if self.pro_opensubtitles_rss:
            log.output("\n[Searching on opensubtitles - rss]")
            self.opensubtitles_rss_results = _opensubtitles.parse_site_results()
        self.opensubtitles_sorted_list = _opensubtitles._sorted_list()

    def _provider_subscene(self) -> None:
        if self.file_exist is False:
            return None
        if self.pro_subscene is False:
            return None
        log.output("\n[Searching on subscene - title]")
        _subscene = subscene.Subscene(self.parameters, self.user_parameters)
        self.subscene_results = _subscene.parse_site_results()
        self.subscene_sorted_list = _subscene._sorted_list()

    def _provider_yifysubtitles(self) -> None:
        if self.file_exist is False:
            return None
        if self.parameters.series:
            log.output("\n[Searching on yifysubtitles - subtitle]]")
            log.output("yifysubtitles only have subtitles for movies")
            log.output("Done with tasks")
            log.output("\n")
            return None
        if self.pro_yifysubtitles:
            log.output("\n[Searching on yifysubtitles - subtitle]")
            _yifysubtitles = yifysubtitles.YifiSubtitles(self.parameters, self.user_parameters)
            self.yifysubtitles_results = _yifysubtitles.parse_site_results()
            self.subscene_sorted_list = _yifysubtitles._sorted_list()

    def _download_files(self) -> None:
        if self.file_exist is False:
            return None
        if self.pro_opensubtitles_hash and self.opensubtitles_hash_results is not None:
            log.output("\n[Downloading from opensubtitles - hash]")
            for item in self.opensubtitles_hash_results:
                self.opensubtitles_hash_downloads = file_manager.download_subtitle(item)
                log.output("Done with tasks")
                log.output("\n")
        if self.pro_opensubtitles_rss and self.opensubtitles_rss_results is not None:
            log.output("\n[Downloading from opensubtitles - rss]")
            for item in self.opensubtitles_rss_results:
                self.opensubtitles_rss_downloads = file_manager.download_subtitle(item)
                log.output("Done with tasks")
        if self.pro_subscene and self.subscene_results is not None:
            log.output("\n[Downloading from subscene]")
            for item in self.subscene_results:
                self.subscene_downloads = file_manager.download_subtitle(item)
                log.output("Done with tasks")
        if self.pro_yifysubtitles and self.yifysubtitles_results is not None:
            log.output("\n[Downloading from yifysubtitles]")
            for item in self.yifysubtitles_results:
                self.yifysubtitles_downloads = file_manager.download_subtitle(item)
                log.output("Done with tasks")

    def _not_downloaded(self) -> None:
        if self.file_exist is False:
            return None
        total_dls = (
            self.opensubtitles_hash_downloads
            + self.opensubtitles_rss_downloads
            + self.subscene_downloads
            + self.yifysubtitles_downloads
        )
        if self.show_download_window and total_dls == 0:
            self.combined_list: list[FormattedData] = (
                self.opensubtitles_sorted_list + self.subscene_sorted_list + self.yifysubtitles_sorted_list
            )
        if len(self.combined_list) > 0:
            widget_menu.open_tab("download", formatted_data=self.combined_list)
            self.ran_download_tab = True

    def _extract_zip_files(self) -> None:
        if self.file_exist is False:
            return None
        if self.ran_download_tab:
            return None
        if self.all_providers_disabled():
            return None
        log.output("\n[Proccessing downloads]")
        file_manager.extract_files(__video__.tmp_directory, __video__.subs_directory, ".zip")

    def _clean_up(self) -> None:
        if self.file_exist is False:
            return None
        if self.rename_best_match:
            file_manager.rename_best_match(f"{self.parameters.release}.srt", __video__.directory, ".srt")
        log.output("\n[Cleaning up]")
        file_manager.clean_up_files(__video__.subs_directory, "nfo")
        file_manager.del_directory(__video__.tmp_directory)
        log.output("Done with tasks")

    def _pre_exit(self) -> None:
        elapsed = time.perf_counter() - self.start
        log.output(f"\nFinished in {elapsed} seconds")

        if self.show_terminal and current_user.check_is_exe() is False:
            try:
                input("Ctrl + c or Enter to exit")
            except KeyboardInterrupt:
                pass



import os
import tkinter as tk
from tkinter import Label, StringVar, ttk

from subsearch.data import __icon__, __tabs__, __titlebar__, __version__, __video__
from subsearch.gui import tk_data
from subsearch.utils import file_manager, raw_config

TKCOLOR = tk_data.Color()
TKFONT = tk_data.Font()
TKWINDOW = tk_data.Window()

GWL_EXSTYLE = -20
WS_EX_APPWINDOW = 0x00040000
WS_EX_TOOLWINDOW = 0x00000080


def get_titlebar_png(btn: str):
    return os.path.join(__titlebar__, btn)


def get_tab_png(tab: str):
    return os.path.join(__tabs__, tab)


def calculate_btn_size(_widget, _width=18, _height=2):
    generic_btn = tk.Button(_widget, width=_width, height=_height)
    x, y = generic_btn.winfo_reqwidth(), generic_btn.winfo_reqheight()
    return x, y


def calculate_checkbtn_size(_widget, _width=16):
    generic_checkbtn = ttk.Checkbutton(_widget, width=_width)
    return generic_checkbtn.winfo_reqwidth()


def set_default_grid_size(_widget, _width=18):
    btn_size = tk.Button(_widget, width=_width, height=2)
    x, y = btn_size.winfo_reqwidth(), btn_size.winfo_reqheight()
    col_count, row_count = _widget.grid_size()
    for col in range(col_count):
        _widget.grid_columnconfigure(col, minsize=x)

    for row in range(row_count):
        _widget.grid_rowconfigure(row, minsize=0)


def asset_tab(_widget, img, type, x=27, y=27):
    path = get_tab_png(f"{img}_{type}.png")
    png = tk.PhotoImage(file=path)
    update_asset(_widget, png, x, y)


def asset_titlebar(_widget, img, type, x=18, y=18):
    path = get_titlebar_png(f"{img}_{type}.png")
    png = tk.PhotoImage(file=path)
    update_asset(_widget, png, x, y)


def asset_icon(_widget, x=18, y=18):
    path = __icon__.replace(".ico", ".png")
    ico = tk.PhotoImage(file=path)
    update_asset(_widget, ico, x, y)


def update_asset(_widget, img, x, y):
    _widget.delete("all")
    _widget.create_image(x, y, image=img)
    _widget.photoimage = img


class TitleBar(tk.Frame):
    def __init__(self, parent, root):
        tk.Frame.__init__(self, parent)
        self.configure(height=37, width=TKWINDOW.width, bg=TKCOLOR.light_black)

        self.root = root
        self.parent = parent
        self.icon = tk.Canvas(
            self,
            width=37,
            height=37,
            bg=TKCOLOR.light_black,
            highlightthickness=0,
        )
        self.tab = tk.Canvas(
            self,
            width=37,
            height=37,
            bg=TKCOLOR.light_black,
            highlightthickness=0,
        )
        self.maximize = tk.Canvas(
            self,
            width=37,
            height=37,
            bg=TKCOLOR.light_black,
            highlightthickness=0,
        )
        self.exit = tk.Canvas(
            self,
            width=37,
            height=37,
            bg=TKCOLOR.light_black,
            highlightthickness=0,
        )

        self.maximize.place(x=TKWINDOW.width - 37, rely=0, anchor="ne")
        self.exit.place(relx=1, rely=0, anchor="ne")
        self.tab.place(x=TKWINDOW.width - 74, rely=0, anchor="ne")
        self.icon.place(x=0, y=0, anchor="nw")

        asset_icon(self.icon)
        asset_titlebar(self.tab, "tab", "rest")
        asset_titlebar(self.maximize, "maximize", "disabled")
        asset_titlebar(self.exit, "exit", "rest")

        self.icon.bind("<Button-1>", self.press_titlebar)
        self.icon.bind("<B1-Motion>", self.drag_titlebar)

        self.bind("<Button-1>", self.press_titlebar)
        self.bind("<B1-Motion>", self.drag_titlebar)

        self.tab.bind("<Enter>", self.enter_event)
        self.exit.bind("<Enter>", self.enter_event)

        self.tab.bind("<Leave>", self.leave_event)
        self.exit.bind("<Leave>", self.leave_event)

    def press_titlebar(self, event):
        self._offsetx = self.winfo_pointerx() - self.winfo_rootx()
        self._offsety = self.winfo_pointery() - self.winfo_rooty()

    def drag_titlebar(self, event):
        x = self.winfo_pointerx() - self._offsetx
        y = self.winfo_pointery() - self._offsety
        self.parent.geometry(f"+{x}+{y}")

    def release_event(self, event):
        if event.widget == self.tab:
            self.parent.wm_withdraw()
            self.root.wm_state("iconic")
        if event.widget == self.exit:
            self.root.destroy()

    def press_event(self, event):
        if event.widget == self.tab:
            self.tab.configure(bg=TKCOLOR.light_grey)
            self.tab.bind("<ButtonRelease-1>", self.release_event)
        if event.widget == self.exit:
            self.exit.configure(bg=TKCOLOR.dark_red)
            self.exit.bind("<ButtonRelease-1>", self.release_event)

    def enter_event(self, event):
        if event.widget == self.tab:
            self.tab.configure(bg=TKCOLOR.dark_grey)
            asset_titlebar(self.tab, "tab", "hover")
            self.tab.bind("<ButtonPress-1>", self.press_event)
        if event.widget == self.exit:
            self.exit.configure(bg=TKCOLOR.red)
            asset_titlebar(self.exit, "exit", "hover")
            self.exit.bind("<ButtonPress-1>", self.press_event)

    def leave_event(self, event):
        if event.widget == self.tab:
            self.tab.configure(bg=TKCOLOR.light_black)
            asset_titlebar(self.tab, "tab", "rest")
            self.tab.unbind("<ButtonRelease-1>")
        if event.widget == self.exit:
            self.exit.configure(bg=TKCOLOR.light_black)
            asset_titlebar(self.exit, "exit", "rest")
            self.exit.unbind("<ButtonRelease-1>")


class CustomBorder(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        csx = TKWINDOW.width
        csy = TKWINDOW.height
        self.canvas_border = tk.Canvas(parent, width=csx, height=csy, bg=TKCOLOR.light_black, borderwidth=0)
        self.canvas_border.place(relx=0.5, rely=0.5, anchor="center")
        self.canvas_bg = tk.Canvas(
            parent,
            width=csx - 4,
            height=csy - 4,
            bg=TKCOLOR.dark_grey,
            highlightthickness=0,
        )
        self.canvas_bg.place(relx=0.5, rely=0.5, anchor="center")

        self.configure(bg=TKCOLOR.light_black)


class WindowPosition(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)

    def set(self, w=TKWINDOW.width, h=TKWINDOW.height, ws_value_offset=0, hs_value_offset=0, other: bool = False):
        ws = self.winfo_screenwidth() + ws_value_offset
        hs = self.winfo_screenheight() + hs_value_offset
        x = int((ws / 2) - (w / 2))
        y = int((hs / 2) - (h / 2))
        value = f"{w}x{h}+{x}+{y}"
        if other:
            return w, h, x, y
        return value


class VarColorPicker:
    def __init__(self, string_var: StringVar, clabel: Label, is_pct: bool = False):
        self.string_var = string_var
        self.clabel = clabel
        self.is_pct = is_pct
        self.pick()

    def pick(self):  # string boolean
        if self.string_var.get() == "True":
            self.clabel.configure(fg=TKCOLOR.green)
        elif self.string_var.get() == "False":
            self.clabel.configure(fg=TKCOLOR.red)
        elif self.string_var.get() == "Both":
            self.clabel.configure(fg=TKCOLOR.blue)
        elif self.string_var.get().startswith("Only"):
            self.clabel.configure(fg=TKCOLOR.green)

        if self.is_pct:
            _pct = raw_config.get_config_key("percentage")
            if _pct in range(75, 101):
                self.clabel.configure(fg=TKCOLOR.green)
            elif _pct in range(50, 75):
                self.clabel.configure(fg=TKCOLOR.green_brown)
            elif _pct in range(25, 50):
                self.clabel.configure(fg=TKCOLOR.red_brown)
            elif _pct in range(0, 25):
                self.clabel.configure(fg=TKCOLOR.red)


class ToolTip(tk.Toplevel):
    def __init__(self, parent, _widget, *_text, _background=TKCOLOR.light_black):
        self.parent = parent
        self.widget = _widget
        self.text = _text
        self.background = _background

    def show(self):
        tk.Toplevel.__init__(self, self.parent)
        self.configure(background=TKCOLOR.light_black)
        # remove the standard window titlebar from the tooltip
        self.overrideredirect(True)
        # unpack *args and put each /n on a new line
        lines = "\n".join(self.text)
        frame = tk.Frame(self, background=TKCOLOR.light_black)
        label = tk.Label(
            frame,
            text=lines,
            background=self.background,
            foreground=TKCOLOR.white_grey,
            justify="left",
        )
        # get size of the label to use later for positioning and sizing of the tooltip
        x, y = label.winfo_reqwidth(), label.winfo_reqheight()
        # set the size of the tooltip background to be 1px larger than the label
        frame.configure(width=x + 1, height=y + 1)

        widget_pos = self.widget.winfo_rootx()
        widget_width = self.widget.winfo_reqwidth()
        widget_center = round(widget_width / 2)
        btn_pos_middle = widget_pos + widget_center
        frame_width = frame.winfo_reqwidth()
        frame_center = round(frame_width / 2)
        center_tip_over_mouse = btn_pos_middle - frame_center
        # offset the frame 1px from edge of the tooltip corner
        frame.place(x=1, y=1, anchor="nw")
        label.place(x=0, y=0, anchor="nw")
        root_x = center_tip_over_mouse  # offset the tooltip by extra 4px so it doesn't overlap the widget
        root_y = self.widget.winfo_rooty() - self.widget.winfo_height() - 4
        # set position of the tooltip, size and add 2px around the tooltip for a 1px border
        self.geometry(f"{x+2}x{y+2}+{root_x}+{root_y}")

    def hide(self):
        self.destroy()



import time
from typing import NamedTuple

import cloudscraper
from bs4 import BeautifulSoup
from bs4.element import Tag

from subsearch.utils import log
from subsearch.utils.exceptions import CaptchaError
from subsearch.utils.raw_config import UserParameters
from subsearch.utils.string_parser import FileSearchParameters

SCRAPER = cloudscraper.create_scraper(browser={"browser": "chrome", "platform": "android", "desktop": False})


class DownloadData(NamedTuple):
    name: str
    file_path: str
    url: str
    idx_num: int
    idx_lenght: int


class FormattedData(NamedTuple):
    provider: str
    release: str
    url: str
    pct_result: str
    formatted_release: str
    formatted_url: str


class BaseProvider:
    """
    Base class for providers
    """

    def __init__(self, parameters: FileSearchParameters, user_parameters: UserParameters):
        # file parameters
        self.parameters = parameters
        self.user_parameters = user_parameters
        self.url_subscene = parameters.url_subscene
        self.url_opensubtitles = parameters.url_opensubtitles
        self.url_opensubtitles_hash = parameters.url_opensubtitles_hash
        self.url_yifysubtitles = parameters.url_yifysubtitles
        self.title = parameters.title
        self.year = parameters.year
        self.season = parameters.season
        self.season_ordinal = parameters.season_ordinal
        self.episode = parameters.episode
        self.episode_ordinal = parameters.episode_ordinal
        self.series = parameters.series
        self.release = parameters.release
        self.group = parameters.group
        self.file_hash = parameters.file_hash
        self.definitive_match = parameters.definitive_match
        # user parameters
        self.current_language = user_parameters.current_language
        self.hi_sub = user_parameters.hearing_impaired
        self.non_hi_sub = user_parameters.non_hearing_impaired
        self.pct_threashold = user_parameters.percentage
        self.show_download_window = user_parameters.show_download_window

    def is_movie(self, key: str) -> bool:
        if key.lower() == f"{self.title} ({self.year})":
            log.output(f"Movie {key} found")
            return True
        return False

    def try_the_year_before(self, key: str) -> bool:
        # Some releases are released close to the next year. If so, the year might differ from the year in the title.
        # This function subtracts one year from the year in the title and checks if the release is in the list.
        if self.year == 0000:
            return False
        _year = int(self.year) - 1
        the_year_before = f"{self.title} ({_year})"
        if key.lower().startswith(the_year_before):
            log.output(f"Movie {key} found")
            return True
        return False

    def is_series(self, key: str) -> bool:
        if self.title and self.season_ordinal in key.lower() and self.series:
            log.output(f"TV-Series {key} found")
            return True
        return False

    def is_threshold_met(self, key: str, pct_result: int) -> bool:
        if pct_result >= self.pct_threashold or self.title and f"{self.season}{self.episode}" in key.lower() and self.series:
            return True
        return False

    def is_subtitle_hearing_impaired(self, a1: Tag) -> bool | None:
        a1_parent = a1.parent
        regular_subtitle = a1_parent.find("td", class_="a40")  # regular
        hearing_impaired_subtitle = a1_parent.find("td", class_="a41")  # hearing impaired
        if regular_subtitle is not None:
            return False
        elif hearing_impaired_subtitle is not None:
            return True
        return None

    def to_many_requests(self):
        time.sleep(1)
        log.output(f"ToManyRequestsWarning: To many requests, sleept for 1 second")

    def is_captcha(self, doc: Tag):
        tag_h2 = doc.find("h2", text="Why do I have to complete a CAPTCHA?")
        if tag_h2 is not None:
            raise CaptchaError("Captcha protection detected. Please try again later.")


def get_lxml_doc(url: str, features: str = "lxml") -> BeautifulSoup:
    source = SCRAPER.get(url)
    scontent = source.content
    doc = BeautifulSoup(scontent, features)
    return doc


def named_tuple_zip_data(provider: str, video_tmp_directory: str, to_be_downloaded: dict[str, str]) -> list[DownloadData]:
    download_info = []
    tbd_lenght = len(to_be_downloaded)
    for zip_idx, (zip_name, zip_url) in enumerate(to_be_downloaded.items(), start=1):
        zip_fp = f"{video_tmp_directory}\\{provider}_{zip_idx}.zip"
        data = DownloadData(name=zip_name, file_path=zip_fp, url=zip_url, idx_num=zip_idx, idx_lenght=tbd_lenght)
        download_info.append(data)
    return download_info


def format_key_value_pct(_provider: str, key: str, value: str, _pct_result: int) -> FormattedData:
    lenght_str = sum(1 for char in f"[{_pct_result:>3}%  match]:")
    number_of_spaces = " " * lenght_str
    _match_release = f"[{_pct_result:>3}%  match]: {key}"
    _url = f"{number_of_spaces} {value}"
    data = FormattedData(
        provider=_provider,
        release=key,
        url=value,
        pct_result=_pct_result,
        formatted_release=_match_release,
        formatted_url=_url,
    )
    return data


def log_and_sort_list(_provider: str, _list: list[FormattedData], pct: int) -> list[FormattedData]:
    _list.sort(key=lambda x: x.pct_result, reverse=True)
    log.output(f"\n[Sorted List from {_provider}]", False)
    hbd_printed = False
    hnbd_printed = False
    for data in _list:
        if data.pct_result >= pct and not hbd_printed:
            log.output(f"--- Has been downloaded ---\n", False)
            hbd_printed = True
        if data.pct_result <= pct and not hnbd_printed:
            log.output(f"--- Has not been downloaded ---\n", False)
            hnbd_printed = True
        log.output(f"{data.formatted_release}", False)
        log.output(f"{data.formatted_url}\n", False)
    return _list



from bs4 import BeautifulSoup

from subsearch.data import __video__
from subsearch.providers import generic
from subsearch.providers.generic import SCRAPER, BaseProvider, FormattedData
from subsearch.utils import log, string_parser
from subsearch.utils.raw_config import UserParameters
from subsearch.utils.string_parser import FileSearchParameters


class Subscene(BaseProvider):
    def __init__(self, parameters: FileSearchParameters, user_parameters: UserParameters):
        BaseProvider.__init__(self, parameters, user_parameters)
        self.scrape = SubsceneScrape()
        self.logged_and_sorted: list[FormattedData] = []

    def parse_site_results(self):
        to_be_scraped: list[str] = []
        definitive_match = self.url_subscene.rsplit("query=", 1)[-1].replace("%20", " ")
        title_keys = self.scrape.get_title(self.url_subscene, definitive_match)
        for release, subtitle_url in title_keys.items():
            if self.is_movie(release):
                to_be_scraped.append(subtitle_url) if subtitle_url not in (to_be_scraped) else None
            if self.try_the_year_before(release):
                to_be_scraped.append(subtitle_url) if subtitle_url not in (to_be_scraped) else None
            if self.is_series(release):
                to_be_scraped.append(subtitle_url) if subtitle_url not in (to_be_scraped) else None
        log.output("Done with task\n") if len(to_be_scraped) > 0 else None

        # exit if no titles found
        if len(to_be_scraped) == 0:
            if self.series:
                log.output(f"\nNo TV-series found matching {self.title}")
            else:
                log.output(f"\nNo movies found matching {self.title}")
            return None

        # search title for subtitles
        to_be_sorted: list[FormattedData] = []
        for subtitle_url in to_be_scraped:
            log.output(f"[Searching on subscene - subtitle]")
            subtitle_data = self.scrape.get_subtitle(self.current_language, self.hi_sub, self.non_hi_sub, subtitle_url)
            break

        _to_be_downloaded: dict[str, str] = {}
        for release, subtitle_url in subtitle_data.items():
            pct_result = string_parser.get_pct_value(release, self.release)
            log.output(f"[{pct_result:>3}%  match]: {release}")
            formatted_data = generic.format_key_value_pct("subscene", release, subtitle_url, pct_result)
            to_be_sorted.append(formatted_data)
            if self.is_threshold_met(release, pct_result) is False:
                continue
            if release in _to_be_downloaded.keys():
                continue
            _to_be_downloaded[release] = subtitle_url
        to_be_scraped.pop(0) if len(to_be_scraped) > 0 else None
        self.logged_and_sorted = generic.log_and_sort_list("subscene", to_be_sorted, self.pct_threashold)
        # exit if no subtitles found
        if len(_to_be_downloaded) == 0:
            log.output(f"No subtitles to download for {self.release}")
            log.output("Done with tasks\n")
            return None

        to_be_downloaded: dict[str, str] = {}
        for release, subtitle_url in _to_be_downloaded.items():
            zip_url = self.scrape.get_download_url(subtitle_url)
            to_be_downloaded[release] = zip_url

        download_info = generic.named_tuple_zip_data("subscene", __video__.tmp_directory, to_be_downloaded)
        log.output("Done with tasks\n")
        return download_info

    def _sorted_list(self):
        return self.logged_and_sorted


class SubsceneScrape(Subscene):
    def __init__(self):
        ...

    def get_title(self, url: str, definitive_match: str) -> dict[str, str]:
        titles: dict[str, str] = {}
        doc = generic.get_lxml_doc(url)
        tag_div = doc.find("div", class_="search-result")
        self.is_captcha(tag_div)
        tag_div = tag_div.find_all("div", class_="title")
        for class_title in tag_div:
            title_name = class_title.contents[1].contents[0].strip().replace(":", "")
            title_url = class_title.contents[1].attrs["href"]
            titles[title_name] = f"https://subscene.com{title_url}"
            if title_name.lower() == definitive_match:
                break
        return titles

    def get_subtitle(self, current_language: str, hi_sub: bool, regular_sub: bool, url: str) -> dict[str, str]:
        subtitles: dict[str, str] = {}
        doc = generic.get_lxml_doc(url)
        tag_tbody = doc.find("tbody")
        while tag_tbody is None:
            self.to_many_requests()
            tag_tbody = doc.find("tbody")
        tag_td = tag_tbody.find_all("td", class_="a1")
        for class_a1 in tag_td:
            is_sub_hi = self.is_subtitle_hearing_impaired(class_a1)
            subtitle_language = class_a1.contents[1].contents[1].contents[0].strip()
            if hi_sub and regular_sub is False and is_sub_hi is False:
                continue
            if hi_sub is False and regular_sub and is_sub_hi:
                continue
            if current_language.lower() != subtitle_language.lower():
                continue
            release_name = class_a1.contents[1].contents[3].string.strip()
            release_name = release_name.replace(" ", ".")
            subtitle_url = class_a1.contents[1].attrs["href"]
            subtitles[release_name] = f"https://subscene.com{subtitle_url}"
        return subtitles

    def get_download_url(self, url: str) -> str:
        source = SCRAPER.get(url).text
        doc = BeautifulSoup(source, "lxml")
        button_url = [dl["href"] for dl in doc.find_all("a", href=True, id="downloadButton")]
        download_url = f"https://subscene.com/{button_url[0]}"
        return download_url



import re
from dataclasses import dataclass
from typing import Any, Literal, Union

import imdb
from num2words import num2words

from subsearch.utils.raw_config import UserParameters


def find_year(string: str) -> int:
    re_year = re.findall("^.*\.([1-2][0-9]{3})\.", string)  # https://regex101.com/r/r5TwxJ/1
    if len(re_year) > 0:
        year = re_year[0]
        return int(year)
    return 0000


def find_title_by_year(string: str) -> str:
    re_title = re.findall("^(.*)\.[1-2][0-9]{3}\.", string)  # https://regex101.com/r/FKUpY0/1
    if len(re_title) > 0:
        title: str = re_title[0]
        title = title.replace(".", " ")
        return title
    return "N/A"


def find_title_by_show(string: str) -> str:
    re_title = re.findall("^(.*)\.[s]\d*[e]\d*\.", string)  # https://regex101.com/r/41OZE5/1
    if len(re_title) > 0:
        title: str = re_title[0]
        title = title.replace(".", " ")
        return title
    return "N/A"


def find_season_episode(string: str) -> str:
    re_se = re.findall("\.([s]\d*[e]\d*)\.", string)  # https://regex101.com/r/8Nwlr6/1
    if len(re_se) > 0:
        se: str = re_se[0]
        return se
    return "N/A"


NA = tuple[Literal["N/A"], Literal["N/A"], Literal["N/A"], Literal["N/A"], Literal[False]]


def find_ordinal(string: str) -> Union[tuple[str, str, str, str, bool], NA]:
    if string == "N/A":
        season, season_ordinal, episode, episode_ordinal = "N/A", "N/A", "N/A", "N/A"
        show_bool = False

        return season, season_ordinal, episode, episode_ordinal, show_bool
    else:
        season, episode = string.replace("s", "").replace("e", " ").split(" ")
        season_ordinal = num2words(int(season), lang="en", to="ordinal")
        episode_ordinal = num2words(int(episode), lang="en", to="ordinal")
        show_bool = True
        return season, season_ordinal, episode, episode_ordinal, show_bool


def find_group(string: str) -> str:
    group = string.rsplit("-", 1)[-1]
    return group


def find_imdb_tt_id(base_yts: str, title: str, year: int) -> str:
    ia = imdb.Cinemagoer()
    movies = ia.search_movie(title)
    url_yifysubtitles = base_yts
    for movie in movies:
        movie_no_colon = movie.data["title"].replace(":", "").split("(")[0]
        if movie_no_colon.lower() == title.lower() and movie.data["year"] == (year or year - 1):
            _movie_id: str = movie.movieID
            tt_id = f"tt{_movie_id}"
            url_yifysubtitles = f"{base_yts}/{tt_id}"
            break
    return url_yifysubtitles


def rpl_sp_pct20(x: str) -> str:
    return x.replace(" ", "%20")


@dataclass(frozen=True, order=True)
class FileSearchParameters:
    url_subscene: str
    url_opensubtitles: str
    url_opensubtitles_hash: str
    url_yifysubtitles: str
    title: str
    year: int
    season: str
    season_ordinal: str
    episode: str
    episode_ordinal: str
    series: bool
    release: str
    group: str
    file_hash: str
    definitive_match: str


def get_parameters(filename: str, file_hash: str, user_parameters: UserParameters) -> FileSearchParameters:
    """
    Parse filename and get parameters for searching on subscene and opensubtitles
    Uses regex expressions to find the parameters

    Args:
        file_name (str): name of the file
        file_hash (str): hash of the file
        lang_abbr (str): language abbreviation for ordinal numbers

    Returns:
        SearchParameters: title, year, season, season_ordinal, episode, episode_ordinal, tv_series, release, group
    """
    filename = filename.lower()

    lang_code3 = user_parameters.languages[user_parameters.current_language]
    year = find_year(filename)
    season_episode = find_season_episode(filename)
    season, season_ordinal, episode, episode_ordinal, series = find_ordinal(season_episode)

    if year != 0000:
        title = find_title_by_year(filename)
    elif series and year == 0000:
        title = find_title_by_show(filename)
    else:
        title = filename.rsplit("-", 1)[0]

    group = find_group(filename)

    base_ss = "https://subscene.com/subtitles/searchbytitle?query="
    base_yts = "https://yifysubtitles.org/movie-imdb"

    if user_parameters.hearing_impaired and user_parameters.non_hearing_impaired is False:
        base_os = f"https://www.opensubtitles.org/en/search/sublanguageid-{lang_code3}/hearingimpaired-on"
    else:
        base_os = f"https://www.opensubtitles.org/en/search/sublanguageid-{lang_code3}"

    url_opensubtitles_hash = f"{base_os}/moviehash-{file_hash}"

    if series:
        url_subscene = f"{base_ss}{title} - {season_ordinal} season"
        url_opensubtitles = f"{base_os}/searchonlytvseries-on/season-{season}/episode-{episode}/moviename-{title}/rss_2_00"
        url_yifysubtitles = base_yts
    else:
        url_subscene = f"{base_ss}{title} ({year})"
        url_opensubtitles = f"{base_os}/searchonlymovies-on/moviename-{title} ({year})/rss_2_00"
        url_yifysubtitles = find_imdb_tt_id(base_yts, title, year)

    definitive_match = url_subscene.rsplit("query=", 1)[-1]

    url_subscene = url_subscene.replace(" ", "%20")
    url_opensubtitles = url_opensubtitles.replace(" ", "%20")

    parameters = FileSearchParameters(
        url_subscene,
        url_opensubtitles,
        url_opensubtitles_hash,
        url_yifysubtitles,
        title,
        year,
        season,
        season_ordinal,
        episode,
        episode_ordinal,
        series,
        filename,
        group,
        file_hash,
        definitive_match,
    )
    return parameters


def get_pct_value(from_pc: Any, from_browser: Any) -> int:
    """
    Parse two lists and return match in %

    Args:
        from_pc (str | int): list from pc
        from_browser (str | int): list from browser
    Returns:
        int: value in percentage of how many words match
    """
    max_percentage = 100
    pc_list: list[Any] = mk_lst(from_pc)
    browser_list: list[Any] = mk_lst(from_browser)
    not_matching = list(set(pc_list) - set(browser_list))
    not_matching_value = len(not_matching)
    number_of_items = len(pc_list)
    percentage_to_remove = round(not_matching_value / number_of_items * max_percentage)
    pct = round(max_percentage - percentage_to_remove)

    return pct


def mk_lst(release: Any) -> list[Any]:
    # create list from string
    new: list[Any] = []
    qualities = ["720p", "1080p", "1440p", "2160p"]
    temp = release.split(".")

    for item in temp:
        if item not in qualities:
            new.append(item.lower())
    return new



import os
import sys
from dataclasses import dataclass
from itertools import product
from typing import Any


@dataclass(order=True)
class VideoFileData:
    name: str | Any
    ext: str | Any
    path: str | Any
    directory: str | Any
    subs_directory: str | Any
    tmp_directory: str | Any


def get_video_file_data() -> VideoFileData:

    """
    Set path, name, directory and ext for the video file

    Returns:
        VideoFilePaths: name, ext, path, directory
    """
    exts = [
        ".avi",
        ".mp4",
        ".mkv",
        ".mpg",
        ".mpeg",
        ".mov",
        ".rm",
        ".vob",
        ".wmv",
        ".flv",
        ".3gp",
        ".3g2",
        ".swf",
        ".mswmm",
    ]
    file_exist = False
    for i in product(exts, sys.argv):
        if i[1].endswith(i[0]) and str(i[1])[i[1].rfind("\\") :].startswith("\\"):
            path = i[1]
            directory = path[: path.rfind("\\")]
            tmp_directory = os.path.join(directory, ".subsearch")
            subs_directory = os.path.join(directory, "subs")
            name = path[path.rfind("\\") + 1 :].rsplit(".", 1)[0]
            ext = i[0]
            file_exist = True
            break

    if file_exist:
        return VideoFileData(name, ext, path, directory, subs_directory, tmp_directory)
    else:
        return VideoFileData(name=None, ext=None, path=None, directory=None, subs_directory=None, tmp_directory=None)
